package com.bokecc.livemodule.localplay;


/**
 * 回放直播间信息监听
 */
public interface DWLocalReplayRoomListener {

    /**
     * 回放播放初始化已经完成
     * @param playerDuration 播放时长
     */
    void videoPrepared(long playerDuration);


    /**
     * 播放完成
     */
    void onPlayComplete();

    /**
     * 播放错误
     *
     * @param code code
     */
    void onPlayError(int code);
    /**
     * seek完成
     */
    void onSeekComplete();

    /**
     * 开始卡顿
     */
    void onBufferStart();

    /**
     * 开始播放
     */
    void onPlay();

    void updateRoomTitle(String title);
}
